@@ -1,4 +1,4 @@
-/* $Id: axconfig.c,v 1.18 2003/07/07 22:29:50 matts Exp $ */
+/* $Id: axconfig.c,v 1.19 2004/07/13 18:05:45 nachbaur Exp $ */
 
 #ifndef WIN32
 #include <modules/perl/mod_perl.h>
@@ -104,6 +104,7 @@ new_axkit_dir_config (pool *p)
     new->config_reader_module = 0;
     new->contentprovider_module = 0;
     new->styleprovider_module = 0;
+    new->dir_handler_module = 0;
     new->default_style = 0;
     new->default_media = 0;
     new->cache_module = 0;
@@ -429,6 +430,13 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
         new->trace_intermediate = ap_pstrdup(p, parent_dir->trace_intermediate);
     }
 
+    if (subdir->dir_handler_module) {
+        new->dir_handler_module = ap_pstrdup(p, subdir->dir_handler_module);
+    }
+    else if (parent_dir->dir_handler_module) {
+        new->dir_handler_module = ap_pstrdup(p, parent_dir->dir_handler_module);
+    }
+
     new->debug_tidy =
         subdir->debug_tidy != -1 ? subdir->debug_tidy :
                                      parent_dir->debug_tidy;
@@ -1338,7 +1346,12 @@ static command_rec axkit_mod_cmds[] = {
     { "AxHandleDirs", ap_set_flag_slot,
       (void *)XtOffsetOf(axkit_dir_config, handle_dirs),
       OR_ALL, FLAG,
-      "On or Off [default] to make AxKit process directory requests using XML::Directory" },
+      "On or Off [default] to make AxKit produce XML for directory requests" },
+
+    { "AxDirHandler", ax_set_module_slot,
+      (void *)XtOffsetOf(axkit_dir_config, dir_handler_module),
+      OR_ALL, TAKE1,
+      "alternative module to use for generating directory xml" },
 
     { "AxIgnoreStylePI", ap_set_flag_slot,
       (void *)XtOffsetOf(axkit_dir_config, ignore_style_pi),
@@ -1460,6 +1473,10 @@ ax_get_config (axkit_dir_config * cfg)
         hv_store(retval, "HandleDirs",
                 10, (newSViv(cfg->handle_dirs)), 0);
     }
+    if (cfg->dir_handler_module) {
+        hv_store(retval, "DirHandler",
+                10, (newSVpv(cfg->dir_handler_module, 0)), 0);
+    }
 
     if (cfg->dependency_checks != -1) {
         hv_store(retval, "DependencyChecks",
