@@ -1,4 +1,19 @@
-# $Id: Cache.pm,v 1.5.2.1 2002/06/06 20:00:19 matts Exp $
+# Copyright 2001-2005 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+
+# $Id: Cache.pm,v 1.14 2005/10/24 18:41:45 matts Exp $
 
 package Apache::AxKit::Cache;
 use strict;
@@ -9,6 +24,8 @@ use Apache::AxKit::Exception;
 use Digest::MD5 ();
 use Compress::Zlib qw(gzopen);
 use Fcntl qw(:flock O_RDWR O_WRONLY O_CREAT O_RDONLY);
+BEGIN { $INC{'bytes.pm'}++ if $] < 5.006 }
+use bytes;
 
 # use vars qw/$COUNT/;
 
@@ -25,7 +42,7 @@ sub new {
     local $^W; # suppress "Use of uninitialized value" warnings
     my $key = Digest::MD5->new->add(
             join(':', 
-                $r->get_server_name,
+                $r->hostname,
                 $r->get_server_port,
                 $xmlfile,
                 @extras
@@ -107,10 +124,11 @@ sub _get_stats {
 sub write {
     my $self = shift;
     return if $self->{no_cache};
+    my $encoding = $_[1] || 'raw';
     AxKit::Debug(7, "[Cache] writing cache file $self->{file}");
     my $fh = Apache->gensym();
     my $tmp_filename = $self->{file}."new$$";
-    if (sysopen($fh, $tmp_filename, O_WRONLY|O_CREAT)) {
+    if (AxKit::sysopen($fh, $tmp_filename, O_WRONLY|O_CREAT, $encoding)) {
         # flock($fh, LOCK_EX);
         # seek($fh, 0, 0);
         # truncate($fh, 0);
@@ -140,8 +158,9 @@ sub write {
 sub read {
     my $self = shift;
     return if $self->{no_cache};
+    my $encoding = $_[0] || 'raw';
     my $fh = Apache->gensym();
-    if (sysopen($fh, $self->{file}, O_RDONLY)) {
+    if (AxKit::sysopen($fh, $self->{file}, O_RDONLY, $encoding)) {
         flock($fh, LOCK_SH);
         local $/;
         return <$fh>;
@@ -152,10 +171,10 @@ sub read {
 }
 
 sub get_fh {
-    my $self = shift;
+    my ($self, $gzip) = @_;
     return if $self->{no_cache};
     my $fh = Apache->gensym();
-    if (sysopen($fh, $self->{file}, O_RDONLY)) {
+    if (AxKit::sysopen($fh, $self->{file} . ($gzip ? '.gz' : ''), O_RDONLY, 'raw')) {
         flock($fh, LOCK_SH);
         return $fh;
     }
@@ -169,7 +188,7 @@ sub set_type {
     return if $self->{no_cache};
     
     my $fh = Apache->gensym();
-    if (sysopen($fh, $self->{file}.'newtype', O_RDWR|O_CREAT)) {
+    if (AxKit::sysopen($fh, $self->{file}.'newtype', O_RDWR|O_CREAT, 'raw')) {
         flock($fh, LOCK_EX);
         seek($fh, 0, 0);
         truncate($fh, 0);
@@ -187,7 +206,7 @@ sub get_type {
     my $self = shift;
     return if $self->{no_cache};
     my $fh = Apache->gensym();
-    if (sysopen($fh, $self->{file}.'.type', O_RDONLY)) {
+    if (AxKit::sysopen($fh, $self->{file}.'.type', O_RDONLY, 'raw')) {
         flock($fh, LOCK_SH);
         local $/;
         return <$fh>;
@@ -210,7 +229,7 @@ sub deliver {
             $r->content_type($type);
         }
     }
-    
+
     if ($r->content_type eq 'changeme' && !$r->notes('axkit_passthru_type')) {
         $AxKit::Cfg->AllowOutputCharset(1);
         $r->content_type('text/html; charset=' . ($AxKit::Cfg->OutputCharset || "UTF-8"));
@@ -219,14 +238,13 @@ sub deliver {
         $r->content_type($AxKit::OrigType);
     }
 
-    
+
     my ($transformer, $doit) = AxKit::get_output_transformer();
-    
+
     if ($doit) {
         AxKit::Debug(4, "Cache: Transforming content and printing to browser");
-        $r->send_http_header() unless lc($r->dir_config('Filter')) eq 'on';
-        $r->print( $transformer->( $self->read() ) );
-        return OK;
+        $r->pnotes('xml_string',$self->read());
+        return OK; # upstream deliver_to_browser should handle the rest
     }
     else {
         AxKit::Debug(4, "Cache: Sending untransformed content to browser");
@@ -234,7 +252,7 @@ sub deliver {
         # Make sure we unset PATH_INFO or wierd things can happen!
         $ENV{PATH_INFO} = '';
         $r->path_info('');
-        
+
         if ($self->{gzip} && $AxKit::Cfg->DoGzip) {
             AxKit::Debug(4, 'Cache: Delivering gzipped output');
             $r->filename($self->{file}.'.gz');
@@ -242,10 +260,10 @@ sub deliver {
         else {
             $r->filename($self->{file});
         }
-        
+
         return DECLINED;
     }
-    
+
 }
 
 sub reset {
