@@ -1,4 +1,19 @@
-# $Id: ConfigReader.pm,v 1.8 2002/06/04 17:21:30 jwalt Exp $
+# Copyright 2001-2005 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+
+# $Id: ConfigReader.pm,v 1.21 2005/07/14 18:43:33 matts Exp $
 
 package Apache::AxKit::ConfigReader;
 
@@ -9,13 +24,13 @@ use strict;
 sub new {
     my $class = shift;
     my $r = shift;
-    
+
 #    my $cfg = AxKit::get_config($r);
 #    if (!$cfg) {
 #        $cfg = {};
 #        AxKit::Debug(2, "Unable to get_config(). Using blank hashref instead");
 #    }
-    
+
 #    use Apache::Peek 'Dump';
 #    Dump($cfg);
 #     use Data::Dumper;
@@ -23,9 +38,9 @@ sub new {
 #     warn("Cfg: ", Data::Dumper->Dump([$cfg], ['cfg']));
 
     my $self = bless { apache => $r, output_charset_ok => 0 }, $class;
-    
+
     $self->get_config($r);
-    
+
     if (my $alternate = $self->{cfg}->{ConfigReader} || $r->dir_config('AxConfigReader')) {
         if ($alternate ne __PACKAGE__) {
             AxKit::reconsecrate($self, $alternate);
@@ -33,14 +48,14 @@ sub new {
             $self->get_config($r);
         }
     }
-    
+
     return $self;
 }
 
 # you may want to override this in your subclass if you write your own ConfigReader
 sub get_config {
     my $self = shift;
-    $self->{cfg} = _get_config($self->{apache});
+        $self->{cfg} = _get_config($self->{apache});
 }
 
 # sub DESTROY {
@@ -61,10 +76,13 @@ sub StyleMap {
 # returns the location of the cache dir
 sub CacheDir {
     my $self = shift;
-    if (my $cachedir = 
-            $self->{cfg}->{CacheDir} 
+    if (my $cachedir =
+            $self->{cfg}->{CacheDir}
             ||
             $self->{apache}->dir_config('AxCacheDir')) {
+        if (substr($cachedir,0,1) eq '+') {
+                $self->{cfg}->{CacheDir} = $cachedir = Apache->request()->document_root.'/'.substr($cachedir,1);
+        }
         return $cachedir;
     }
     
@@ -138,8 +156,8 @@ sub CacheModule {
 
 sub DebugLevel {
     my $self = shift;
-    return $self->{cfg}{DebugLevel} || 
-            $self->{apache}->dir_config('AxDebugLevel') || 
+    return $self->{cfg}{DebugLevel} ||
+            $self->{apache}->dir_config('AxDebugLevel') ||
             0;
 }
 
@@ -160,12 +178,22 @@ sub TraceIntermediate {
     if (my $dir = $self->{cfg}{TraceIntermediate} ||
             $self->{apache}->dir_config('AxTraceIntermediate')) {
         return undef if $dir =~ m/^\s*(?:off|none|disabled?)\s*$/i;
+        if (substr($dir,0,1) eq '+') {
+                $self->{cfg}{TraceIntermediate} = $dir = Apache->request()->document_root.'/'.substr($dir,1);
+        }
         return $dir;
     }
 
     return undef;
 }
 
+sub DebugTidy {
+    my $self = shift;
+    return $self->{cfg}{DebugTidy} ||
+            $self->{apache}->dir_config('AxDebugTidy') ||
+            0;
+}
+
 sub LogDeclines {
     my $self = shift;
     return $self->{cfg}{LogDeclines} ||
@@ -180,6 +208,18 @@ sub HandleDirs {
             0;
 }
 
+sub DirHandlerClass {
+    my $self = shift;
+    if (my $alternate = $self->{cfg}{DirHandler} || 
+            $self->{apache}->dir_config('AxDirHandler')) {
+        AxKit::load_module($alternate);
+        return $alternate;
+    }
+    
+    AxKit::load_module('Apache::AxKit::DirHandler::Basic');
+    return 'Apache::AxKit::DirHandler::Basic';
+}
+
 sub IgnoreStylePI {
     my $self = shift;
     return $self->{cfg}{IgnoreStylePI} ||
@@ -201,7 +241,7 @@ sub OutputCharset {
     my $self = shift;
 
     return unless $self->{output_charset_ok};
-    
+
 #    warn "OutputCharset\n";
     unless ($self->{cfg}{TranslateOutput} ||
             $self->{apache}->dir_config('AxTranslateOutput')) {
@@ -217,7 +257,7 @@ sub OutputCharset {
     
 #    warn "Checking Accept-Charset\n";
     # check HTTP_ACCEPT_CHARSET
-    if (my $ok_charsets = $ENV{HTTP_ACCEPT_CHARSET}) {
+    if (my $ok_charsets = $self->{apache}->header_in('Accept-Charset')) {
         my @charsets = split(/,\s*/, $ok_charsets);
         my $retcharset;
         my $retscore = 0;
@@ -235,18 +275,23 @@ sub OutputCharset {
         
         $retcharset =~ s/iso/ISO/;
         $retcharset =~ s/(us\-)?ascii/US-ASCII/;
-        
+
         return undef if $retcharset =~ /^utf\-?8$/;
 	return undef if $retcharset eq '*';
 # warn "Charset: '$retcharset'\n";
         return $retcharset;
     }
-    
+
+}
+
+sub ExternalEncoding {
+    my $self = shift;
+    return $self->{cfg}{ExternalEncoding} || "UTF-8";
 }
 
 sub ErrorStyles {
     my $self = shift;
-    
+
     my $style = $self->{cfg}{ErrorStylesheet};
     my ($type, $href);
     if (!$style || !@$style) {
@@ -285,8 +330,38 @@ sub DoGzip {
     
     AxKit::Debug(5, 'Should we zip the output?');
     my $r = $self->{apache};
-    my($can_gzip);
+    return unless $r->header_in('Accept-Encoding') =~ /gzip/io;
+    my $can_gzip;
     
+    return if $r->protocol =~ /http\/1\.0/i;
+
+    my $ua = $r->header_in('User-Agent');
+    if ($ua =~ /MSIE 4\./) {
+        return if 
+            $r->method =~ /POST/i ||
+            $r->header_in('Range') ||
+            length($r->uri) > 245;
+    }
+    elsif ($ua =~ /MSIE 6\.0/) {
+        # Don't gzip https on MSIE 6.0
+        return if $r->parsed_uri->scheme =~ /https/i;
+    }
+
+    # All too buggy to gzip
+    if ($r->header_in('Via') =~ /^1\.1\s/ || # MS Proxy 2.0
+        $r->header_in('Via') =~ /^Squid\// ||
+        $ua =~ /Galean\)/ ||
+        $ua =~ /Mozilla\/4\.7[89]/ ||
+        $ua =~ /Opera 3\.5/ ||
+        $ua =~ /SkipStone\)/)
+    {
+        return;
+    }
+
+    if (($ua =~ /Mozilla\/4\.0/) and (!($ua =~ /compatible/i))) {
+        return;
+    }
+
     AxKit::Debug(5, 'Getting Vary header');
     my @vary;
     @vary = $r->header_out('Vary') if $r->header_out('Vary');
@@ -296,11 +371,8 @@ sub DoGzip {
                     join ", ",
                     @vary
                 );
-    my($accept_encoding) = $r->header_in("Accept-Encoding") || '';
-    $can_gzip = 1 if index($accept_encoding,"gzip")>=0;
     unless ($can_gzip) {
-        my $user_agent = $r->header_in("User-Agent");
-        if ($user_agent =~ m{
+        if ($ua =~ m{
                              ^Mozilla/
                              \d+
                              \.
@@ -322,6 +394,42 @@ sub DoGzip {
     return $can_gzip;
 }
 
+sub _extract_dir_config_processors {
+    my $self = shift;
+    my ($style, $media) = @_;
+
+    my @list;
+    
+    my $processors = $self->{apache}->dir_config('AxProcessors');
+    my $found_non_global_style = 0;
+    if( $processors ) {
+      foreach my $processor (split(/\s*,\s*/, $processors) ) {
+        my ($pmedia, $pstyle, @processor) = split(/\s+/, $processor);
+        next unless ($pmedia eq $media and ($pstyle eq $style or $pstyle eq '#global'));
+        $found_non_global_style++ if $pstyle ne '#global';
+        push (@list, [ 'NORMAL', @processor ] );
+      }
+    }
+    
+    my @processors = $self->{apache}->dir_config->get('AxProcessor');
+    foreach my $processor (@processors) {
+        my ($pmedia, $pstyle, @processor) = split(/\s+/, $processor);
+        next unless ($pmedia eq $media and ($pstyle eq $style or $pstyle eq '#global'));
+        $found_non_global_style++ if $pstyle ne '#global';
+        push (@list, [ 'NORMAL', @processor ] );
+    }
+
+    return @list if $style eq '#default';
+
+    if ($found_non_global_style) {
+        return @list;
+    }
+    else {
+        return $self->_extract_dir_config_processors('#default', $media);
+    }
+}
+
+
 sub GetMatchingProcessors {
     my $self = shift;
     my ($media, $style, $doctype, $dtd, $root, $styles, $provider) = @_;
@@ -329,16 +437,23 @@ sub GetMatchingProcessors {
     
     $style ||= '#default';
 
-    my $list = $self->{cfg}{Processors}{$media}{$style};
+    # Add global styles
+    my $list = [];
 
-    my $processors = $self->{apache}->dir_config('AxProcessors');
-    if( $processors ) {
-      foreach my $processor (split(/\s*,\s*/, $processors) ) {
-	my ($pmedia, $pstyle, @processor) = split(/\s+/, $processor);
-	next unless ($pmedia eq $media and $pstyle eq $style);
-	push (@$list, [ 'NORMAL', @processor ] );
-      }
+    if( exists($self->{cfg}{Processors}{$media}{'#global'})) {
+	push @$list, @{ $self->{cfg}{Processors}{$media}{'#global'} || []};
     }
+
+    # Add styles matching this style
+    if (exists($self->{cfg}{Processors}{$media}{$style})) {
+        push @$list, @{ $self->{cfg}{Processors}{$media}{$style} || [] };
+    }
+    else {
+        # if the style didn't exist, push the #default styles
+        push @$list, @{ $self->{cfg}{Processors}{$media}{'#default'} || [] };
+    }
+
+    push @$list, $self->_extract_dir_config_processors($style, $media);   
     
     my @results;
     
@@ -349,45 +464,43 @@ sub GetMatchingProcessors {
                     href => $directive->[2],
                     title => $style,
                 };
-        if ($type eq 'NORMAL') {
+        if (lc($type) eq 'normal') {
             push @results, $style_hash;
         }
-        elsif ($type eq 'DocType') {
+        elsif (lc($type) eq 'doctype') {
             if ($doctype eq $directive->[3]) {
                 push @results, $style_hash;
             }
         }
-        elsif ($type eq 'DTD') {
+        elsif (lc($type) eq 'dtd') {
             if ($dtd eq $directive->[3]) {
                 push @results, $style_hash;
             }
         }
-        elsif ($type eq 'Root') {
+        elsif (lc($type) eq 'root') {
             if ($root eq $directive->[3]) {
                 push @results, $style_hash;
             }
         }
-        elsif ($type eq 'URI') {
+        elsif (lc($type) eq 'uri') {
             my $uri = $provider->apache_request->uri;
             if ($uri =~ /$directive->[3]/) {
                 push @results, $style_hash;
             }
         }
+        elsif (lc($type) eq 'dynamic') {
+            my $package = $directive->[3];
+            AxKit::load_module($package);
+            no strict 'refs';
+            my($handler) = $package.'::handler';
+            push @results, $handler->($provider, $media, $style,
+                                      $doctype, $dtd, $root);
+        } 
         else {
             warn "Unrecognised directive type: $type";
         }
     }
-    
-    # list any dynamically chosen stylesheets here
-    $list = $self->{cfg}{DynamicProcessors};
-    foreach my $package (@$list) {
-        AxKit::load_module($package);
-        no strict 'refs';
-        my($handler) = $package.'::handler';
-        push @results, $handler->($provider, $media, $style, 
-                                  $doctype, $dtd, $root);
-    }   
-    
+
     return @results;
 }
 
