@@ -1,4 +1,19 @@
-# $Id: HtmlDoc.pm,v 1.2 2002/05/26 16:47:58 matts Exp $
+# Copyright 2001-2005 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+
+# $Id: HtmlDoc.pm,v 1.6 2005/07/14 18:43:34 matts Exp $
 # Apache::AxKit::Language::HtmlDoc - xhtml->pdf renderer
 package Apache::AxKit::Language::HtmlDoc;
 
@@ -10,8 +25,9 @@ use Apache;
 use Apache::Constants qw(:common);
 use Apache::Request;
 use Apache::AxKit::Language;
-use Apache::AxKit::Language::LibXSLT;
+use Apache::AxKit::LibXMLSupport;
 use Apache::AxKit::Provider;
+use XML::LibXSLT;
 use IPC::Run qw(run);
 use Cwd;
 
@@ -25,10 +41,9 @@ sub handler {
     my ($r, $xml_provider, undef, $last_in_chain) = @_;
 
     my $parser = XML::LibXML->new();
-    local $XML::LibXML::match_cb = \&Apache::AxKit::Language::LibXSLT::match_uri;
-    local $XML::LibXML::open_cb = \&Apache::AxKit::Language::LibXSLT::open_content_uri;
-    local $XML::LibXML::read_cb = \&Apache::AxKit::Language::LibXSLT::read_uri;
-    local $XML::LibXML::close_cb = \&Apache::AxKit::Language::LibXSLT::close_uri;
+    local($XML::LibXML::match_cb, $XML::LibXML::open_cb,
+        $XML::LibXML::read_cb, $XML::LibXML::close_cb);
+    Apache::AxKit::LibXMLSupport->reset();
 
     my $dom;
     my $source_text;
@@ -68,9 +83,9 @@ EOX
     $input =~ s{ href="(?!/|.{0,5}:)}{ href="http://$host$path}g;
     AxKit::Debug(8, "About to shell out to htmldoc - hope you have it installed...");
     AxKit::Debug(10, $input);
-    my $rc = run(['htmldoc','--quiet','--format','pdf13','--truetype','--size','a4','--color','--charset','8859-15','--webpage',$r->dir_config->get('AxHtmlDocOptions'),'-'],\$input,\$result);
+    run(['htmldoc','--quiet','--format','pdf13','--truetype','--size','a4','--color','--charset','8859-15','--webpage',$r->dir_config->get('AxHtmlDocOptions'),'-'],\$input,\$result);
 
-    if (!$rc) {
+    if (substr($result,0,5) ne '%PDF-') {
         throw Apache::AxKit::Exception::Error(-text => 'htmldoc returned error: '.$result);
     }
 
