@@ -1,4 +1,19 @@
-# $Id: LibXSLT.pm,v 1.13 2002/06/04 07:52:17 matts Exp $
+# Copyright 2001-2005 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+
+# $Id: LibXSLT.pm,v 1.24 2005/08/10 12:21:56 matts Exp $
 
 package Apache::AxKit::Language::LibXSLT;
 
@@ -10,6 +25,7 @@ use Apache;
 use Apache::Request;
 use Apache::AxKit::Language;
 use Apache::AxKit::Provider;
+use Apache::AxKit::LibXMLSupport;
 use File::Basename qw(dirname);
 
 @ISA = 'Apache::AxKit::Language';
@@ -48,20 +64,19 @@ sub handler {
     
     my $parser = XML::LibXML->new();
     $parser->expand_entities(1);
-    local $XML::LibXML::match_cb = \&match_uri;
-    local $XML::LibXML::open_cb = \&open_content_uri;
-    local $XML::LibXML::read_cb = \&read_uri;
-    local $XML::LibXML::close_cb = \&close_uri;
+    local($XML::LibXML::match_cb, $XML::LibXML::open_cb,
+          $XML::LibXML::read_cb, $XML::LibXML::close_cb);
+    Apache::AxKit::LibXMLSupport->reset();
+    local $Apache::AxKit::LibXMLSupport::provider_cb = 
+        sub {
+            my $r = shift;
+            my $provider = Apache::AxKit::Provider->new_content_provider($r);
+            add_depends($provider->key());
+            return $provider;
+        };
 
     if (!$xml_doc && !$xmlstring) {
-        eval {
-            my $fh = $xml->get_fh();
-            $xml_doc = $parser->parse_fh($fh, $r->uri());
-        };
-        if ($@) {
-            $xmlstring = ${$xml->get_strref()};
-            $xml_doc = $parser->parse_string($xmlstring, $r->uri());
-        }
+        $xml_doc = $xml->get_dom();
     } 
     elsif ($xmlstring) {
         $xml_doc = $parser->parse_string($xmlstring, $r->uri());
@@ -73,7 +88,7 @@ sub handler {
 
     my $stylesheet;
     my $cache = $style_cache{$style->key()};
-    if ($cache && !$style->has_changed($cache->{mtime}) && ref($cache->{depends}) eq 'ARRAY') {
+    if (ref($cache) eq 'HASH' && !$style->has_changed($cache->{mtime}) && ref($cache->{depends}) eq 'ARRAY') {
         AxKit::Debug(8, "[LibXSLT] checking if stylesheet is cached");
         my $changed = 0;
         DEPENDS:
@@ -91,25 +106,28 @@ sub handler {
     }
     
     if (!$stylesheet || ref($stylesheet) ne 'XML::LibXSLT::Stylesheet') {
-        my $style_doc;
         reset_depends();
-        my $style_uri = $style->apache_request->uri();
+        my $style_uri = $style->get_document_uri();
         AxKit::Debug(7, "[LibXSLT] parsing stylesheet $style_uri");
-        eval {
-            my $fh = $style->get_fh();
-            $style_doc = $parser->parse_fh($fh, $style_uri);
-        };
-        if ($@) {
-            my $stylestring = $style->get_strref();
-            $style_doc = $parser->parse_string($$stylestring, $style_uri);
-        }
-        
-        local $XML::LibXML::open_cb = \&open_stylesheet_uri;
+        my $style_doc = $style->get_dom();
         
+        local($XML::LibXML::match_cb, $XML::LibXML::open_cb,
+            $XML::LibXML::read_cb, $XML::LibXML::close_cb);
+        Apache::AxKit::LibXMLSupport->reset();
+        local $Apache::AxKit::LibXMLSupport::provider_cb = 
+            sub {
+                my $r = shift;
+                my $provider = Apache::AxKit::Provider->new_style_provider($r);
+                add_depends($provider->key());
+                return $provider;
+            };
+    
         $stylesheet = XML::LibXSLT->parse_stylesheet($style_doc);
         
-        $style_cache{$style->key()} = 
+        unless ($r->dir_config('AxDisableXSLTStylesheetCache')) {
+            $style_cache{$style->key()} = 
                 { style => $stylesheet, mtime => time, depends => [ get_depends() ] };
+        }
     }
 
     # get request form/querystring parameters
@@ -119,16 +137,21 @@ sub handler {
 
     my $results = $stylesheet->transform($xml_doc, @params);
     
-    if ($last_in_chain && $XML::LibXSLT::VERSION >= 1.03) {
-        my $encoding = $stylesheet->output_encoding;
-        my $type = $stylesheet->media_type;
-        $r->content_type("$type; charset=$encoding");
+    AxKit::Debug(7, "[LibXSLT] transformation finished, creating $results");
+    
+    if ($last_in_chain) {
+        AxKit::Debug(8, "[LibXSLT] outputting to \$r");
+        if ($XML::LibXSLT::VERSION >= 1.03) {
+            my $encoding = $stylesheet->output_encoding;
+            my $type = $stylesheet->media_type;
+            $r->content_type("$type; charset=$encoding");
+        }
+        $stylesheet->output_fh($results, $r);
     }
 
-    $stylesheet->output_fh($results, $r) if $last_in_chain;
-
+    AxKit::Debug(7, "[LibXSLT] storing results in pnotes(dom_tree) ($r)");
     $r->pnotes('dom_tree', $results);
-
+    
 #         warn "LibXSLT returned $output \n";
 #         print $stylesheet->output_string($results);
     return Apache::Constants::OK;
@@ -145,68 +168,4 @@ sub fixup_params {
     return @results;
 }
 
-sub match_uri {
-    my $uri = shift;
-    AxKit::Debug(8, "LibXSLT match_uri: $uri");
-    return 1 if $uri =~ /^axkit:/;
-    return $uri !~ /^\w+:/; # only handle URI's without a scheme
-}
-
-sub open_content_uri {
-    my $uri = shift || './';
-    AxKit::Debug(8, "LibXSLT open_content_uri: $uri");
-    
-    if ($uri =~ /^axkit:/) {
-        return AxKit::get_axkit_uri($uri);
-    }
-    
-    # create a subrequest, so we get the right AxKit::Cfg for the URI
-    my $apache = AxKit::Apache->request;
-    my $sub = $apache->lookup_uri($uri);
-    local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
-    
-    my $provider = Apache::AxKit::Provider->new_content_provider($sub);
-    
-    add_depends($provider->key());
-    my $str = $provider->get_strref;
-    
-    undef $provider;
-    undef $apache;
-    undef $sub;
-    
-    return $$str;
-}
-
-sub open_stylesheet_uri {
-    my $uri = shift || './';
-    AxKit::Debug(8, "LibXSLT open_stylesheet_uri: $uri");
-    
-    if ($uri =~ /^axkit:/) {
-        return AxKit::get_axkit_uri($uri);
-    }
-    
-    # create a subrequest, so we get the right AxKit::Cfg for the URI
-    my $apache = AxKit::Apache->request;
-    my $sub = $apache->lookup_uri($uri);
-    local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
-    
-    my $provider = Apache::AxKit::Provider->new_style_provider($sub);
-    
-    add_depends($provider->key());
-    my $str = $provider->get_strref;
-    
-    undef $provider;
-    undef $apache;
-    undef $sub;
-    
-    return $$str;
-}
-
-sub close_uri {
-}
-
-sub read_uri {
-    return substr($_[0], 0, $_[1], "");
-}
-
 1;
