@@ -1,4 +1,19 @@
-# $Id: PassiveTeX.pm,v 1.1 2002/01/13 20:45:11 matts Exp $
+# Copyright 2001-2005 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+
+# $Id: PassiveTeX.pm,v 1.4 2005/07/14 18:43:34 matts Exp $
 
 package Apache::AxKit::Language::PassiveTeX;
 
@@ -66,28 +81,30 @@ sub handler {
             close($fh) || fail("Cannot close: $!");
         }
     }
-    
+
     chdir($tempdir) || fail("Cannot cd: $!");
     
-    local $ENV{TEXINPUTS} = dirname($r->filename()); 
+    local $ENV{TEXINPUTS} = dirname($r->filename()) . ":";
     AxKit::Debug(8, "About to shell out to pdfxmltex - hope you have passivetex installed...");
-    my $retval = system("pdfxmltex temp.fo");
+    my $retval = system("pdfxmltex --interaction=batchmode --shell-escape temp.fo");
     $retval >>= 8;
-    
+
     if ($retval) {
         fail("pdfxmltex exited with $retval");
     }
-    
+
     my $pdfh = Apache->gensym();
     open($pdfh, "temp.pdf") || fail("Cannot open PDF: $!");
-    
+
     $AxKit::Cfg->AllowOutputCharset(0);
-    
+
     $r->content_type("application/pdf");
-    
+
     local $/;
-    
+
     $r->print(<$pdfh>);
+
+    return Apache::Constants::OK;
 }
 
 sub cleanup {
