@@ -1,4 +1,19 @@
-# $Id: XPathScript.pm,v 1.7 2002/05/31 19:22:24 matts Exp $
+# Copyright 2001-2005 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+
+# $Id: XPathScript.pm,v 1.15 2005/08/09 16:13:08 matts Exp $
 
 package Apache::AxKit::Language::XPathScript;
 
@@ -17,6 +32,7 @@ use Apache::AxKit::Language;
 use Apache::AxKit::Cache;
 use Apache::AxKit::Exception;
 use Apache::AxKit::CharsetConv;
+use bytes;
 
 @ISA = 'Apache::AxKit::Language';
 
@@ -154,12 +170,16 @@ sub check_inc_mtime {
     
     for my $inc (@$includes) {
 #        warn "Checking mtime for $inc\n";
-        my $sub = $apache->lookup_uri($inc);
-        local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
+        my $sub = $apache->lookup_uri(AxKit::FromUTF8($inc));
+        my $cfg = Apache::AxKit::ConfigReader->new($sub);
         
-        my $inc_provider = Apache::AxKit::Provider->new_style_provider($sub);
+        my $changed = do {
+            local $AxKit::Cfg = $cfg;
+            my $inc_provider = Apache::AxKit::Provider->new_style_provider($sub);
+            $inc_provider->has_changed($mtime);
+        };
         
-        if ($inc_provider->has_changed($mtime)) {
+        if ($changed) {
 #            warn "$inc newer (" . $inc_provider->mtime() . ") than last compile ($mtime) causing recompile\n";
             return;
         }
@@ -266,6 +286,7 @@ sub compile {
             '; use Apache qw(exit);',
             'use XML::XPath::Node;',
             'Apache::AxKit::Language::XPathScript::Toys->import;',
+            'use bytes;',
             'sub handler {',
             'my ($r, $xp, $t) = @_;',
             "\n#line 1 " . $provider->key() . "\n",
@@ -286,16 +307,19 @@ sub compile {
 }
 
 sub include_file {
-    my ($filename, $provider, $script_output) = @_;
+    my ($filename, $provider, $script_output, $ignore_cache) = @_;
 
-    # return if already included
     my $key = $provider->key();
-    return '' if grep {$_ eq $filename} @{$stash->{$key}{includes}};
 
+    unless ($ignore_cache) {
+        # return if already included
+        return '' if grep {$_ eq $filename} @{$stash->{$key}{includes}};
+    }
+    
     push @{$stash->{$key}{includes}}, $filename;
     
     my $apache = $provider->apache_request;
-    my $sub = $apache->lookup_uri($filename);
+    my $sub = $apache->lookup_uri(AxKit::FromUTF8($filename));
     local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
     
     my $inc_provider = Apache::AxKit::Provider->new_style_provider($sub);
@@ -323,6 +347,8 @@ sub XML::XPath::Function::document {
     my $newdoc;
     if ($uri =~ /^axkit:/) {
         $newdoc = $parser->parse( AxKit::get_axkit_uri($uri) );
+    } elsif ($uri =~ /^xmldb:/) {
+        $newdoc = $parser->parse( Apache::AxKit::Provider::XMLDB::get_xmldb_uri($uri) );
     }
     elsif ($uri =~ /^\w\w+:/) { # assume it's scheme://foo uri
         eval {
@@ -348,7 +374,7 @@ sub XML::XPath::Function::document {
         
         # create a subrequest, so we get the right AxKit::Cfg for the URI
         my $apache = AxKit::Apache->request;
-        my $sub = $apache->lookup_uri($uri);
+        my $sub = $apache->lookup_uri(AxKit::FromUTF8($uri));
         local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
         
         my $provider = Apache::AxKit::Provider->new_content_provider($sub);
@@ -383,7 +409,7 @@ sub get_mtime {
     
     for my $inc (@{$stash->{$filename}{includes}}) {
         
-        my $sub = $apache->lookup_uri($inc);
+        my $sub = $apache->lookup_uri(AxKit::FromUTF8($inc));
         local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
         
         my $inc_provider = Apache::AxKit::Provider->new_style_provider(
@@ -447,7 +473,7 @@ sub get_package_name {
     
     sub import_template {
         my ($filename, $local_changes) = @_;
-        my ($script) = Apache::AxKit::Language::XPathScript::include_file($filename,$Apache::AxKit::Language::XPathScript::style_provider, 1);
+        my ($script) = Apache::AxKit::Language::XPathScript::include_file($filename,$Apache::AxKit::Language::XPathScript::style_provider, 1, 1);
         # changes may be local to this imported template, or global (default).
         my ($setup_t);
         if ($local_changes) {
