@@ -1,4 +1,19 @@
-# $Id: TaglibHelper.pm,v 1.1 2002/01/13 20:45:12 matts Exp $
+# Copyright 2001-2005 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+
+# $Id: TaglibHelper.pm,v 1.6 2005/07/14 18:43:34 matts Exp $
 
 package Apache::AxKit::Language::XSP::TaglibHelper;
 @ISA = qw(Apache::AxKit::Language::XSP);
@@ -75,10 +90,12 @@ sub optional_args ($) {
 
     my ($argspec) = ($funspec =~ /; *([^\)]*)/);
     my @retval;
-    foreach my $arg(split (/,/, $argspec)) {
-        $arg =~ s/^\s*//g;
-        $arg =~ s/\s*$//g;
-        push (@retval, $arg);
+    if ($argspec) {
+       foreach my $arg (split (/,/, $argspec)) {
+           $arg =~ s/^\s*//g;
+           $arg =~ s/\s*$//g;
+           push (@retval, $arg);
+       }
     }
     return @retval;
 }
@@ -279,7 +296,7 @@ sub parse_start {
     # arbitrarily here.
     $tag =~ s/-/_/g;
 
-    my $pkg = caller;
+    my $pkg = $AxKit::XSP::TaglibPkg;
 
     # horrible hack: if the caller is the SAX library directly,
     # then we'll just have to assume that we're testing TaglibHelper
@@ -396,7 +413,7 @@ sub parse_end {
     my $origtag = $tag;
     $tag =~ s/-/_/g;
 
-    my $pkg = caller;
+    my $pkg = $AxKit::XSP::TaglibPkg;
     $pkg = "Apache::AxKit::Language::XSP::TaglibHelper" if $pkg eq "AxKit::XSP::SAXHandler";
     my $funspec = is_function($pkg, $tag);
 
@@ -515,27 +532,38 @@ TaglibHelper - module to make it easier to write a taglib
 
 =head1 SYNOPSIS
 
-Put this code at the top of your taglib module:
+    package My::Taglib;
 
-    # this stuff, you change for each taglib
+    use Apache::AxKit::Language::XSP::TaglibHelper;
+
+    @ISA = qw( Apache::AxKit::Language::XSP::TaglibHelper );
+
+    ## Edit $NS to be the namespace URI you want
     $NS = 'http://apache.org/xsp/testtaglib/v1';
+
+    ## Edit @EXPORT_TAGLIB as needed
     @EXPORT_TAGLIB = (
-    'func1($arg1)',
-    'func2($arg1,$arg2)',
-    'func3($arg1,$arg2;$optarg)',
-    'func4($arg1,*treearg)',
-    'func4($arg1,*treearg):listtag=mylist:itemtag=item',
+        'func1($arg1)',
+        'func2($arg1,$arg2)',
+        'func3($arg1,$arg2;$optarg)',
+        'func4($arg1,*treearg)',
+        'func4($arg1,*treearg):listtag=mylist:itemtag=item',
     );
 
-    # this stuff you don't
-    use Apache::AxKit::Language::XSP::TaglibHelper;
-    sub parse_char { Apache::AxKit::Language::XSP::TaglibHelper::parse_char(@_); }
-    sub parse_start { Apache::AxKit::Language::XSP::TaglibHelper::parse_start(@_); }
-    sub parse_end { Apache::AxKit::Language::XSP::TaglibHelper::parse_end(@_); }
     use strict;
 
-...and then edit the $NS and @EXPORT_TAGLIB to reflect
-your taglib's namespace and list of functions.
+    sub func1 {
+        my ( $arg1 ) = @_ ;
+        ...
+        return $scalar_or_reference;
+    }
+
+    ...
+
+    1;
+
+
+the functions with the same names as listed in C<@EXPORT_TAGLIB>.
 
 =head1 DESCRIPTION
 
