@@ -1,27 +1,112 @@
-# $Id: Provider.pm,v 1.9 2002/05/31 19:22:23 matts Exp $
+# Copyright 2001-2005 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+
+# $Id: Provider.pm,v 1.24 2005/08/10 12:21:55 matts Exp $
 
 package Apache::AxKit::Provider;
 use strict;
 
 use Apache::AxKit::Exception;
 use Apache::Constants qw(OK DECLINED);
-#use XML::Parser;
+use AxKit;
 
 # use vars qw/$COUNT/;
 
+# protocol registry
+
+my %pseudo_protocols =
+  (
+   'axkit' => \&AxKit::get_axkit_uri,
+   );
+
+sub register_protocol {
+    my ($protocol, $handler) = @_;
+    $pseudo_protocols{lc($protocol)} = $handler;
+}
+
+sub has_protocol {
+    exists $pseudo_protocols{lc($_[0])};
+}
+
+# end of protocol registry
+
+sub get_provider {
+    my ($uri,$r,$provider_cb) = @_;
+    AxKit::Debug(8,"get_provider: $uri");
+    my ($protocol) = ($uri =~ m/^([a-zA-Z0-9]+):/);
+    if ($protocol) {
+        throw Apache::AxKit::Exception::Error(-text => "this resource has no provider ($uri)") if !ref($pseudo_protocols{$protocol});
+        return $pseudo_protocols{$protocol}->new_provider($r, uri => $uri);
+    } elsif (substr($uri,0,1) ne '/') { # resolve relative URIs -- FIXME: does $r->lookup_uri do this for us?
+        my $base = AxKit::ToUTF8($r->uri());
+        $base =~ s{[^/]*$}{};
+        $uri = $base.$uri;
+        $uri =~ s/\?.*$//;
+        $uri =~ s{/./}{/}g;
+        $uri =~ s{/[^/]+/+../}{/}g;
+        AxKit::Debug(8,"Absolute URI: ".$uri);
+    }
+    
+    # create a subrequest, so we get the right AxKit::Cfg for the URI
+    my $sub = $r->lookup_uri($uri);
+    my $cfg = Apache::AxKit::ConfigReader->new($sub);
+    
+    my $result = do {
+        local $AxKit::Cfg = $cfg;
+        ($provider_cb?$provider_cb->($sub):__PACKAGE__->new_content_provider($sub));
+    };
+    undef $sub;
+    undef $cfg;
+    return $result;
+}
+
+sub get_uri {
+    my ($uri,$r,$provider_cb) = @_;
+    my ($protocol) = ($uri =~ m/^([a-zA-Z0-9]+):/);
+    if ($protocol && ref($pseudo_protocols{$protocol})) {
+        AxKit::Debug(8, "using pseudo_protocol: $protocol");
+        return $pseudo_protocols{$protocol}->($uri,$r);
+    }
+    
+    my $str = get_provider(@_)->get_strref;
+    
+    return $$str;
+}
+
+# end of protocol utilities
+
+sub new_provider { # make this just "new"? (would be logical, but probably breaks things...)
+    my ($self, $r, @args) = @_;
+    my $provider = bless { apache => $r }, $self;
+    $provider->init(@args);
+    return $provider;
+}
+
 sub new_style_provider {
     my $class = shift;
     my $apache = shift;
     my $self = bless { apache => $apache }, $class;
-    
+
     if (my $alternate = $AxKit::Cfg->StyleProviderClass()) {
-        AxKit::Debug(7, "Style Provider Override: $alternate" );
+        AxKit::Debug(8, "Style Provider Override: $alternate" );
         AxKit::reconsecrate($self, $alternate);
     }
-    
+
     $self->init(@_);
-    
-    AxKit::add_depends($self->key());
+
+    AxKit::add_depends("{style}".$self->key());
 
     return $self;
 }
@@ -30,15 +115,15 @@ sub new_content_provider {
     my $class = shift;
     my $apache = shift;
     my $self = bless { apache => $apache }, $class;
-    
+
     if (my $alternate = $AxKit::Cfg->ContentProviderClass()) {
-        AxKit::Debug(7, "Content Provider Override: $alternate" );
+        AxKit::Debug(8, "Content Provider Override: $alternate" );
         AxKit::reconsecrate($self, $alternate);
     }
-    
+
     $self->init(@_);
-    
-    AxKit::add_depends($self->key());
+
+    AxKit::add_depends("{content}".$self->key());
 
     return $self;
 }
@@ -56,6 +141,64 @@ sub init {
 #     AxKit::Debug(7, "Provider->DESTROY Count: " . --$COUNT);
 # }
 
+sub get_strref {
+    throw Apache::AxKit::Exception::Error (
+        -text => "Subclass must implement get_strref"
+        );
+}
+
+sub get_fh {
+    throw Apache::AxKit::Exception::Error (
+        -text => "Subclass must implement get_fh"
+        );
+}
+
+
+sub get_document_uri {
+    return shift->{apache}->uri();
+}
+
+sub get_dom {
+    my ($self, $str) = @_;
+    require Apache::AxKit::LibXMLSupport;
+    
+    AxKit::Debug(8, "Provider::get_dom");
+    my $parser = XML::LibXML->new();
+    $parser->expand_entities(1);
+    $parser->expand_xinclude(1);
+    local($XML::LibXML::match_cb, $XML::LibXML::open_cb,
+          $XML::LibXML::read_cb, $XML::LibXML::close_cb);
+    Apache::AxKit::LibXMLSupport->reset($parser);
+    
+    my $r = $self->apache_request();
+    
+    my $xml_doc;
+    if ($str) {
+    	use bytes;
+        AxKit::Debug(8, "Provider::get_dom/parse_string($str, ", $self->get_document_uri(), ")");
+        $xml_doc = $parser->parse_string($str, $self->get_document_uri()) || 
+            throw Apache::AxKit::Exception::Error(
+                -text => "XML::LibXML->parse_string returned nothing!"
+                );
+	} else {    
+		eval {
+			my $fh = $self->get_fh();
+			AxKit::Debug(8, "Provider::get_dom/parse_fh($fh, ", $self->get_document_uri(), ")");
+			$xml_doc = $parser->parse_fh($fh, $self->get_document_uri());
+		};
+		if ($@) {
+			use bytes;
+			my $xmlstring = ${$self->get_strref()};
+			AxKit::Debug(8, "Provider::get_dom/parse_strref($xmlstring, ", $self->get_document_uri(), ")");
+			$xml_doc = $parser->parse_string($xmlstring, $self->get_document_uri()) || 
+				throw Apache::AxKit::Exception::Error(
+					-text => "XML::LibXML->parse_string returned nothing!"
+					);
+		}
+	}		
+    return $xml_doc;
+}
+
 sub apache_request {
     my $self = shift;
     return $self->{apache};
@@ -85,17 +228,9 @@ sub get_ext_ent_handler {
             if ($pubid) {
                 return ''; # do not bring in public DTD's
             }
-            eval {
-                require HTTP::GHTTP;
-            };
-            if ($@) {
-                require LWP::Simple;
-                import LWP::Simple;
-                return get($sysid) || die "Cannot get $sysid";
-            }
-            my $r = HTTP::GHTTP->new($sysid);
-            $r->process_request;
-            return $r->get_body;
+            require LWP::Simple;
+            import LWP::Simple;
+            return get($sysid) || die "Cannot get $sysid";
         }
         elsif ($sysid =~ /^(https|ftp):/) {
             if ($pubid) {
@@ -106,19 +241,18 @@ sub get_ext_ent_handler {
 
         # create a subrequest, so we get the right AxKit::Cfg for the URI
         my $apache = AxKit::Apache->request;
-        my $sub = $apache->lookup_uri($sysid);
-        local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
+        my $sub = $apache->lookup_uri(AxKit::FromUTF8($sysid));
+        my $cfg = Apache::AxKit::ConfigReader->new($sub);
     
-#        warn "File provider ext_ent_handler called with '$sysid'\n";
-        my $provider = Apache::AxKit::Provider->new($sub);
-        
-#        warn "Got provider with key: ", $provider->key, "\n";
-        my $str = $provider->get_strref;
-#        warn "Returning string with length: ", length($$str), "\n";
+        my $str = do {
+            local $AxKit::Cfg = $cfg;
+            my $provider = Apache::AxKit::Provider->new($sub);
+            $provider->get_strref;
+        };
 
-        undef $provider;
         undef $apache;
         undef $sub;
+        undef $cfg;
         
         return $$str;
     };
@@ -132,9 +266,7 @@ sub get_styles {
         undef $pref_style;
     }
 
-    my $xml_styles = [];
-    my $vals = [];
-
+	my @styles;
     my $key = $self->key();
 
     # need to extract the following from the XML file:
@@ -142,68 +274,25 @@ sub get_styles {
     #   DTD filename
     #   Root element name (including namespace)
     # use three element array @$vals
-
-    if (defined &Apache::AxKit::Provider::xs_get_styles_fh) {
-        AxKit::Debug(2, "using XS get_styles (libxml2)");
-        my ($xs_styles, $doctype, $dtd, $root) =
-                $self->xs_get_styles($media, $pref_style);
-        @$xml_styles = @$xs_styles unless $AxKit::Cfg->IgnoreStylePI();
-        @$vals = ($doctype, $dtd, $root);
-    }
-    else {
-        require XML::Parser;
-
-        AxKit::Debug(4, "get_styles: creating XML::Parser");
-
-        my $handlers = {
-                    Start => \&parse_start,
-                    Doctype => \&parse_dtd,
-                    $AxKit::Cfg->IgnoreStylePI() ? () : (Proc => \&parse_pi),
-                    };
-
-        my $xml_parser = XML::Parser->new(
-                Namespaces => 1,
-                ErrorContext => 2,
-                Handlers => $handlers,
-            );
-
-        my $to_parse;
-        eval {
-            $to_parse = $self->get_fh();
-        };
-        if ($@) {
-            $to_parse = ${ $self->get_strref(); };
-        }
-        
-        AxKit::Debug(4, "get_styles: calling XML::Parser->parse('$key')");
-        $xml_parser->parse(
-                $to_parse,
-                XMLStyle_preferred => $pref_style,
-                XMLStyle_media => $media,
-                XMLStyle_style => $xml_styles,
-                XMLStyle_vals => $vals,
-                XMLStyle_style_screen => [],
-                );
-            
-        AxKit::Debug(4, "get_styles: parse returned successfully");
-    }
-    
-    foreach my $style (@$xml_styles) {
-        $style->{title} ||= '#default';
-    }
-    
-    # Let GetMatchingProcessors to process the @$styles array
-    {
-      local $^W; # suppress "Use of uninitialized value" warnings
-      AxKit::Debug(4, "Calling GetMatchingProcessors with ($media, $pref_style, $vals->[0], $vals->[1], $vals->[2])");
-    }
-    my @styles = $AxKit::Cfg->GetMatchingProcessors($media,
-		$pref_style, @$vals[0 .. 2], $xml_styles, $self);
+	
+	my ($doctype, $dtd, $root, $xml_styles) = $self->get_xml_info( $media, $pref_style );
+	
+	$xml_styles = [] if $AxKit::Cfg->IgnoreStylePI();
+	
+   	foreach my $style (@$xml_styles) {
+   	    $style->{title} ||= '#default';
+   	}
     
+   	# Let GetMatchingProcessors to process the @$styles array
+   	{
+   	  local $^W; # suppress "Use of uninitialized value" warnings
+   	  AxKit::Debug(4, "Calling GetMatchingProcessors with ($media, $pref_style, $doctype, $dtd, $root)");
+   	}
+
+   	@styles = $AxKit::Cfg->GetMatchingProcessors($media, $pref_style, $doctype, $dtd, $root, $xml_styles, $self);
+
     if (!@styles) {
-        throw Apache::AxKit::Exception::Declined(
-                reason => "No styles defined for '$key'"
-                );
+        throw Apache::AxKit::Exception::Declined( reason => "No styles defined for '$key'" );
     }
     
     # get mime-type => module mapping
@@ -235,20 +324,105 @@ sub get_styles {
     return \@styles;
 }
 
+sub get_xml_info{
+	my ($self, $media, $pref_style ) = @_;
+	
+	# This is where the sniffing of the xml content happens.
+		
+	my $r = $self->apache_request();
+	my @ret;
+	eval{
+		my $fh = $self->get_fh();
+		my $pos = eval { tell $fh; };
+		if($@) {
+			# fh is not seekable, thus we must slurp file at this point.
+			undef $@;
+			local($/) = undef;
+			my $string = <$fh>;	
+			$r->pnotes('xml_string', $string);
+			@ret = parse_xml_info( $r, undef, \$string, $media, $pref_style);
+		} else {
+			# seekable.
+			@ret = parse_xml_info( $r, $fh, undef, $media, $pref_style);
+			seek $fh, 0, $pos;
+		}			
+	};
+	if($@) {
+		my $str_ref = $self->get_strref();
+		$r->pnotes('xml_string', ${$str_ref});
+		@ret = parse_xml_info( $r, undef, $str_ref, $media, $pref_style);
+	}
+	return @ret;		
+}
+
+sub parse_xml_info{
+    my ($r, $fh, $str_ref, $media, $pref_style) = @_;
+
+    if ($pref_style eq '#default') {
+        undef $pref_style;
+    }
+
+    my $xml_styles = [];
+    my $vals = [];
+	my @styles;
+
+	if (defined &Apache::AxKit::Provider::xs_get_styles_fh) {
+		AxKit::Debug(2, "using XS get_styles (libxml2)");
+		my ($xs_styles, $doctype, $dtd, $root) = xs_get_styles($r, $fh, $str_ref, $media, $pref_style);
+		@$xml_styles = @$xs_styles unless $AxKit::Cfg->IgnoreStylePI();
+		@$vals = ($doctype, $dtd, $root);
+	}
+	else {
+		require XML::Parser;
+
+		AxKit::Debug(4, "get_styles: creating XML::Parser");
+
+		my $handlers = {
+			Start => \&parse_start,
+			Doctype => \&parse_dtd,
+			$AxKit::Cfg->IgnoreStylePI() ? () : (Proc => \&parse_pi),
+		};
+
+		my $xml_parser = XML::Parser->new(
+			Namespaces => 1,
+			ErrorContext => 2,
+			Handlers => $handlers,
+		);
+
+		my $to_parse = $fh || ${$str_ref};
+		#eval {
+		#	$to_parse = $provider->get_fh();
+		#};
+		#if ($@) {
+		#	$to_parse = ${ $provider->get_strref(); };
+		#}
+		
+		AxKit::Debug(4, "get_styles: calling XML::Parser->parse()");
+		$xml_parser->parse(
+			$to_parse,
+			XMLStyle_preferred => $pref_style,
+			XMLStyle_media => $media,
+			XMLStyle_style => $xml_styles,
+			XMLStyle_vals => $vals,
+			XMLStyle_style_screen => [],
+		);
+			
+		AxKit::Debug(4, "get_styles: parse returned successfully");
+	}
+	
+	return (@$vals, $xml_styles);
+}
+
 sub xs_get_styles {
-    my $self = shift;
-    my ($media, $pref_style) = @_;
+    my ($r, $fh, $strref, $media, $pref_style) = @_;
     
     my $bits;
-    eval {
-        my $fh = $self->get_fh();
+    if(defined($fh)) {
         AxKit::Debug(4, "calling xs_get_styles_fh()");
-        $bits = xs_get_styles_fh($self->apache_request, $fh);
-    };
-    if ($@) {
-        my $strref = $self->get_strref();
+        $bits = xs_get_styles_fh($r, $fh);
+    } else {
         AxKit::Debug(4, "calling xs_get_styles_str()");
-        $bits = xs_get_styles_str($self->apache_request, $$strref);
+        $bits = xs_get_styles_str($r, $$strref);
     }
     
     my @xml_stylesheet = @{$bits->[0]};
@@ -270,7 +444,7 @@ sub xs_get_styles {
         XMLStyle_style_screen => [],
         XMLStyle_preferred => $pref_style,
         XMLStyle_media => $media,
-        };
+    };
     
     foreach my $pi (@xml_stylesheet) {
         parse_pi($e, "xml-stylesheet", $pi);
@@ -441,6 +615,7 @@ sub parse_pi {
     }
 }
 
+
 1;
 __END__
 
@@ -479,7 +654,7 @@ you need to implement:
 
 Determine whether or not to process this URL. For example, you don't want
 to process a directory request, or if the resource doesn't exist. Return 1
-to tell AxKit to process this URL, or die with a Declined exception (with 
+to tell AxKit to process this URL, or die with a Declined exception (with
 a reason) if you do not wish to process this URL.
 
 =head2 mtime()
@@ -490,7 +665,19 @@ Return the last modification time in days before the current time.
 
 Extract the stylesheets from the XML resource. Should return an array
 ref of styles. The style entries are hashes refs with required keys
-'href' and 'type'.
+'href', 'module' and 'type'.
+
+A sample return value of get_styles() could be:
+
+  [{ type => "text/xsl",
+     href => "path/to/your/stylesheet.xsl",
+     module => "Apache::AxKit::Language::LibXSLT" }]
+
+This allows one to use a different stylemapping than the default
+server configuration. If 'module' is ommited AxKit will apply the 
+given default module for the type as it was added by 'AxAddStyleMap'
+in the configuration.
+
 
 =head2 get_fh()
 
