@@ -1,9 +1,24 @@
-# $Id: FileSuffix.pm,v 1.6 2001/01/14 13:18:07 matt Exp $
+# Copyright 2001-2005 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+
+# $Id: FileSuffix.pm,v 1.4 2005/07/14 18:43:35 matts Exp $
 
 package Apache::AxKit::StyleChooser::FileSuffix;
 
 use strict;
-use Apache::Constants;
+use Apache::Constants qw(OK);
 use Apache::URI;
 
 sub handler {
@@ -17,7 +32,7 @@ sub handler {
 #		warn "setting notes: $style\n";
 		$r->notes('preferred_style', $style);
 	}
-	return DECLINED;
+	return OK;
 }
 
 1;
@@ -27,14 +42,18 @@ __END__
 
 Apache::AxKit::StyleChooser::FileSuffix - Choose stylesheet using file suffix
 
+=head1 SYNOPSIS
+
+	PerlTypeHandler Apache::AxKit::StyleChooser::FileSuffix
+	SetHandler AxKit
+
 =head1 DESCRIPTION
 
 This module lets you pick a stylesheet based on the filename suffix. To use
 it, simply add this module to the list of PerlTypeHandlers:
 
 	PerlTypeHandler Apache::AxKit::StyleChooser::FileSuffix
-	SetHandler perl-script
-	PerlHandler AxKit
+	SetHandler AxKit
 
 Then simply by referencing your xml files as follows:
 
